@@ -1,3 +1,4 @@
+# vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
 #
@@ -346,6 +347,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         self.config_box = gtk.VBox()
         self.config_box.set_border_width(gu.PAD)
         self.config_box.show()
+        self.config_box_sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
         if no_notebook:
             self.pack_start(vbox)
             self.pack_start(self.config_box, False)
@@ -537,6 +539,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         but lets see if that can improve.
         """
         self.handle_config_box_visibility()
+        self.handle_statistics_page_sensibility()
     def on_end_practise(self):
         pass
     def handle_config_box_visibility(self):
@@ -549,6 +552,16 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.config_box.show()
         else:
             self.config_box.hide()
+    def handle_statistics_page_sensibility(self):
+        try:
+            if self.m_t.m_custom_mode:
+                self.g_statview.set_sensitive(False)
+                self.g_statview.set_tooltip_text(_("Statistics is disabled. Either because you selected a “Configure yourself” exercise, or because “Expert mode” is selected in the preferences window."))
+            else:
+                self.g_statview.set_sensitive(True)
+                self.g_statview.set_tooltip_text(None)
+        except AttributeError: # not all exercises has g_statview
+            pass
     def on_key_press_event(self, widget, event):
         if (self.g_notebook is None or self.g_notebook.get_current_page() == 0) \
            and event.type == gtk.gdk.KEY_PRESS:
@@ -566,13 +579,10 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         self.g_notebook.connect('switch_page', self.on_switch_page)
     def on_switch_page(self, notebook, obj, pagenum):
         if pagenum == 2:
-            if isinstance(self.m_t, Teacher):
-                if self.m_t.m_P:
-                    self.g_statview.update()
-                else:
-                    self.g_statview.clear()
-            else:
+            if self.m_t.m_P and not self.m_t.m_custom_mode:
                 self.g_statview.update()
+            else:
+                self.g_statview.clear()
     def _add_auto_new_question_gui(self, box):
         hbox = gu.bHBox(box, False)
         hbox.set_spacing(gu.PAD_SMALL)
@@ -687,24 +697,29 @@ class RhythmAddOnGuiClass(object):
         self.g_select_rhythms_box.set_border_width(gu.PAD_SMALL)
         frame.add(self.g_select_rhythms_box)
     def add_select_num_beats_gui(self):
-        table = gtk.Table(2, 4, False)
-        self.config_box.pack_start(table, False)
         ###
+        hbox = gtk.HBox()
+        hbox.set_spacing(gu.hig.SPACE_SMALL)
         label = gtk.Label(_("Number of beats in question:"))
+        hbox.pack_start(label, False)
+        self.config_box_sizegroup.add_widget(label)
         label.set_alignment(1.0, 0.5)
-        table.attach(label, 0, 1, 0, 1, xpadding=gu.PAD_SMALL,
-                      xoptions=gtk.FILL)
-        table.attach(gu.nSpinButton(self.m_exname, "num_beats",
-                     gtk.Adjustment(4, 1, 100, 1, 10)),
-                     1, 2, 0, 1, xoptions=gtk.FILL)
+        hbox.pack_start(gu.nSpinButton(self.m_exname, "num_beats",
+                     gtk.Adjustment(4, 1, 100, 1, 10)), False)
+        self.config_box.pack_start(hbox, False)
+        hbox.show_all()
+        #
+        hbox = gtk.HBox()
+        hbox.set_spacing(gu.hig.SPACE_SMALL)
         label = gtk.Label(_("Count in before question:"))
+        hbox.pack_start(label, False)
+        self.config_box_sizegroup.add_widget(label)
         label.set_alignment(1.0, 0.5)
-        table.attach(label, 0, 1, 1, 2, xpadding=gu.PAD_SMALL,
-                     xoptions=gtk.FILL)
-        table.attach(gu.nSpinButton(self.m_exname, "count_in",
-                     gtk.Adjustment(2, 0, 10, 1, 10)),
-                     1, 2, 1, 2, xoptions=gtk.FILL)
-        table.show_all()
+        hbox.pack_start(label, False)
+        hbox.pack_start(gu.nSpinButton(self.m_exname, "count_in",
+                     gtk.Adjustment(2, 0, 10, 1, 10)), False)
+        hbox.show_all()
+        self.config_box.pack_start(hbox, False)
     def pngcheckbutton(self, i):
         btn = gtk.CheckButton()
         btn.add(gu.create_rhythm_image(const.RHYTHMS[i]))
@@ -753,6 +768,7 @@ class IntervalGui(Gui):
     """
     Creates 'New interval' and 'Repeat' buttons in the action_area.
     """
+    keyboard_accel = 99
     def __init__(self, teacher):
         Gui.__init__(self, teacher)
 
@@ -832,7 +848,7 @@ class IntervalGui(Gui):
                 'minor9', 'major9', 'minor10', 'major10']
         self.m_key_bindings = {}
         for idx in range(len(keys)):
-            self.m_key_bindings['interval_input/'+keys[idx]] = lambda idx=idx, self=self: self.click_on_interval(1, idx+1, None)
+            self.m_key_bindings['interval_input/'+keys[idx]] = lambda idx=idx, self=self: self.click_on_interval(self.keyboard_accel, idx+1, None)
     def repeat_question(self, *w):
         self.m_t.play_question()
         self.g_input.grab_focus_first_sensitive_button()
