@@ -345,12 +345,16 @@ class DB(object):
         else:
             hashvalue, fileid = row
             if hashvalue != cur_lessonfile_hash_value:
-                cursor.execute("delete from sessions where fileid=?", (fileid,))
-                cursor.execute("delete from tests where fileid=?", (fileid,))
-                cursor.execute("update lessonfiles "
-                    "set hash=?, test_passed=?, test_result=0.0 where fileid=?",
-                    (cur_lessonfile_hash_value, None, fileid))
-                self.conn.commit()
+                replaces = solfege.lessonfile.infocache.get(filename, 'replaces')
+                if not isinstance(replaces, list):
+                    replaces = [replaces]
+                if not hashvalue in replaces:
+                    cursor.execute("delete from sessions where fileid=?", (fileid,))
+                    cursor.execute("delete from tests where fileid=?", (fileid,))
+                    cursor.execute("update lessonfiles "
+                        "set hash=?, test_passed=?, test_result=0.0 where fileid=?",
+                        (cur_lessonfile_hash_value, None, fileid))
+                    self.conn.commit()
     def recent(self, count):
         filenames = []
         for fileid, timestamp in self.conn.execute('select fileid, timestamp from sessions '
@@ -408,6 +412,15 @@ class AbstractStatistics(object):
         self.m_t = teacher
         self.m_timestamp = None
         self.m_test_mode = False
+    def int_if_int(self, s):
+        """
+        Return an int if the can be converted to an int.
+        Return unchanged if not.
+        """
+        try:
+            return int(s)
+        except ValueError:
+            return s
     def get_keys(self, all_keys=False):
         """
         Return the keys for all questions that have been answered correctly.
@@ -422,7 +435,9 @@ class AbstractStatistics(object):
             c = solfege.db.conn.execute("select distinct(answerkey) from sessions where fileid=?", (fileid,))
         else:
             c = solfege.db.conn.execute("select distinct(answerkey) from sessions where fileid=? and answerkey=guessed", (fileid,))
-        return [x[0] for x in c]
+        v = [self.int_if_int(x[0]) for x in c]
+        v.sort()
+        return [unicode(x) for x in v]
     def get_statistics(self, seconds):
         """
         return a dict with statistics more recent than 'seconds' seconds.
@@ -443,15 +458,14 @@ class AbstractStatistics(object):
             q = solfege.db.conn.execute("select answerkey, guessed, sum(count) from sessions where fileid=? and timestamp>? group by answerkey, guessed", (fileid, self.m_timestamp - seconds))
         ret = {}
         for answer, guess, count in q.fetchall():
-            ret.setdefault(answer, {})
-            ret[answer][guess] = count
+            ret.setdefault(self.int_if_int(answer), {})
+            ret[self.int_if_int(answer)][self.int_if_int(guess)] = count
         return ret
     def reset_session(self):
         """
         Start a new practise session.
         """
         self.m_timestamp = int(time.time())
-        rows = solfege.db.conn.execute("select * from lessonfiles where filename=?", (self.m_t.m_P.m_filename,)).fetchall()
     def enter_test_mode(self):
         self.m_test_mode = True
     def exit_test_mode(self):
@@ -481,7 +495,7 @@ class AbstractStatistics(object):
         row = cursor.execute(
                 "select count from %s where fileid=? and timestamp=? "
                 "and answerkey=? and guessed=?" % table,
-                (fileid, self.m_timestamp, question, answer)).fetchone()
+                (fileid, self.m_timestamp, unicode(question), unicode(answer))).fetchone()
         if not row:
             cursor.execute(
                 "insert into %s "
@@ -495,7 +509,7 @@ class AbstractStatistics(object):
                 "update %s set count=? where "
                 "fileid=? and timestamp=? and answerkey=? and guessed=?" % table,
                 (row[0] + 1, fileid,
-                 self.m_timestamp, question, answer))
+                 self.m_timestamp, unicode(question), unicode(answer)))
         solfege.db.conn.commit()
     def add_wrong(self, question, answer):
         self._add(question, answer)
